// Telos2 phone interface (Telos2)
var t2Connections=[];

/*
Telos 2 Protocol

available functions:
connection management
 void phoneEU.OpenConnection(cid, string host, string user, string pass, int line);
 void phoneEU.CloseConnection(cid);
 void phoneEU.CloseAllConnections(void);
live operations
 void phoneEU.PostDropButton(cid);
 void phoneEU.PostHoldButton(cid);
 void phoneEU.PostResumeButton(cid);
 void phoneEU.PostLineButton(cid);
 void phoneEU.PostDialKey(cid, int key);
 void phoneEU.CallNumber(cid, string number);
 void phoneEU.PostUpdateAll(cid);

available events:
 ph2Connected(cid);
 ph2Disconnected(cid);
 ph2SendLineState(cid, lineState);
 ph2SendRingLine(cid, callerId);

*/
// Telos2 protocol line states
const t2LineState=
{
	unknown:   0,
	notready:  1, // Line cannot be used to dial out.
	spidpend:  2, // Waiting for SPID confirmation.
	spiderr:   3, // SPID has been rejected
	idle:      4, // Line is not engaged, calls can be placed.
	inring:    5, // Call is ringing in. Please accept or reject.
	accept:    6, // Call accept has been issued (ISDN only). Line should go automatically go to 'connect'.
	connect:   7, // Line is connected.
	disc:      8, // Line disconnect has been issued (ISDN only). Line should got automatically to 'idle'. See 'ended'.
	seizepend: 9, // Outgoing overlap call/seize has been issued.
	seized:   10, // Line seize confirmed (ISDN only), digits can be dialed.
	proceed:  11, // Outgoing call has been accepted by telco (ISDN only).
	outring:  12, // Outgoing call is ringing at far end.
	ended:    13, // Call has been disconnected but B-channel is still active (ISDN only). Different from state 'disc' the call must be actively disconnected to release the B-channel.
	suspend:  14, // Suspended has been issued (ISDN only). It should automatically go to 'idle'.
	resume:   15, // Resume has been issued (ISDN only). It should automatically go to 'connect'.
	sshold:   16, // SS: Hold request
	ssxfer:   17, // SS: Call transfer request
	ssreconn: 18  // SS: Reconnect request
};

// ------------------------- * Telos2 interface entry point * -------------------------

function ph2Connected(cid)
{
	log.info_f("ph2Connected(%d)", cid);
	if(!(cid in t2Connections)) return;

	var t2=t2Connections[cid];
	if(t2.state!='conhost') return;

	t2.state='working';
	t2.lineState='NONE';
	t2.remote='';
	phoneIndiStudio(cid, 0, 0, 1);
	phoneIndiFixedState(cid, 1, t2.lineState, t2.remote);
}
function ph2Disconnected(cid)
{
	log.info_f("ph2Disconnected(%d)", cid);
	if(!(cid in t2Connections)) return;

	var t2=t2Connections[cid];
	if(t2.state=='idle') return;

	t2.state='conhost';
	t2.lineState='NONE';
	t2.remote='';
// TODO: indicate connection loss
	log.info_f("t2%d: Connection lost", cid);
	phoneIndiFixedState(cid, 1, t2.lineState, t2.remote);
}

/*
** ph2SendLineState
*/
function ph2SendLineState(cid, lineState)
{
	log.info_f("ph2SendLineState(%d, %d)", cid, lineState);
	if(!(cid in t2Connections)) return;

	var t2=t2Connections[cid];
	if(t2.state!='working') return;

	if(lineState==t2LineState.notready)
	{
	/* filter short IDLE->NOT-READY transitions */
		t2.filterTO=host.resetTimeout(t2.filterTO, 1000, function(){this.filterTO=null; this.lineState='NONE'; this.remote=''; phoneIndiFixedState(this.id, 1, this.lineState, this.remote);}, t2);
		return;
	}
	t2.filterTO=host.clearTimeout(t2.filterTO);

	switch(lineState)
	{
	case t2LineState.unknown:
	case t2LineState.notready:
	default:
		t2.lineState='NONE';
		t2.remote='';
		break;
	case t2LineState.idle:
		if(t2.lineState=='ON_HOLD' || t2.lineState=='SEIZED' || t2.lineState=='IDLE') return; /* igonre our "virtual" states */
	/* fall through */
	case t2LineState.disc: // should got automatically to 'idle'
		t2.lineState='IDLE';
		t2.remote='';
		break;
	case t2LineState.inring:
	case t2LineState.accept:
		t2.lineState='RINGING_IN';
		break;
	case t2LineState.connect: // On Air [Arrow down]
	case t2LineState.ended:   // disconnected but B-channel is still active
	case t2LineState.resume:  // Goind off hold to connect
		t2.lineState='ON_AIR';
		break;
	case t2LineState.seizepend: // Seizing
	case t2LineState.seized:    // Dialing
	case t2LineState.proceed:   // Proceeding
	case t2LineState.outring:   // Out Ringing
		t2.lineState='ON_AIR_DIALING';
		break;
	case t2LineState.suspend:  // Suspended, will go to idle. Present that as ON_HOLD
		t2.lineState='ON_HOLD';
		break;
	}
	phoneIndiFixedState(cid, 1, t2.lineState, t2.remote);
}

/*
** ph2SendLineState
*/
function ph2SendRingLine(cid, callerId)
{
	log.info_f("ph2SendLineState(%d, %s)", cid, callerId);
	if(!(cid in t2Connections)) return;

	var t2=t2Connections[cid];
	if(t2.state!='working') return;

	t2.lineState='RINGING_IN';
	t2.remote=callerId;
	phoneIndiFixedState(cid, 1, t2.lineState, t2.remote);
}

// ------------------------- * Telos TWO protocol commands * -------------------------

/*
** t2Connect
**
** Connection request from scripts
*/
function t2Connect(cid, host, login, pass, line)
{
	log.info_f("t2Connect(%d, %s, %s, %s, %d)", cid, host, login, pass, line);

	if(cid in t2Connections)
	{
		if(t2Connections[cid].state!='idle') phoneEU.CloseConnection(cid);
		if(t2Connections[cid].filterTO) host.clearTimeout(t2Connections[cid].filterTO);
		delete vxConnections[cid];
	}
	if(host=="" || !phoneEU.OpenConnection(cid, host, login, pass, line))
		return;
	t2Connections[cid]=
	{
		id: cid,
		state: 'conhost', // {idle, conhost, working}
		host: host,
		login:login,
		pass: pass,
		lineState: 'NONE',
		remote: '',
		filterTO: null // linestate filter
	};
}

function t2CmndFixedDrop(cid, hybrid)
{
	if(!(cid in t2Connections)) return;
	var t2=t2Connections[cid];

	if(t2.state!='working') return;
	if(hybrid!=1) return;
	if(t2.lineState=='SEIZED') /* drop our virtual SEIZED state */
	{
		t2.lineState='IDLE';
		phoneIndiFixedState(cid, 1, t2.lineState, t2.remote);
		return;
	}
	phoneEU.PostDropButton(cid);
}

function t2CmndFixedHold(cid, hybrid)
{
	if(!(cid in t2Connections)) return;
	var t2=t2Connections[cid];

	if(t2.state!='working') return;
	if(hybrid!=1) return;
	phoneEU.PostHoldButton(cid);
}

function t2CmndFixedTake(cid, hybrid)
{
	if(!(cid in t2Connections)) return;
	var t2=t2Connections[cid];

	if(t2.state!='working') return;
	if(hybrid!=1) return;
	if(t2.lineState=='ON_HOLD')
		phoneEU.PostResumeButton(cid);
	else
		phoneEU.PostLineButton(cid);
}

function t2CmndFixedSeize(cid, hybrid)
{
	if(!(cid in t2Connections)) return;
	var t2=t2Connections[cid];

	if(t2.state!='working') return;
	if(hybrid!=1) return;
/* T2 doesn't have real SEIZED state, so we need to emulate that */
	if(t2.lineState=='IDLE')
	{
		t2.lineState='SEIZED';
		phoneIndiFixedState(cid, 1, t2.lineState, t2.remote);
	}
}

function t2CmndFixedCall(cid, hybrid, number)
{
	if(!(cid in t2Connections)) return;
	var t2=t2Connections[cid];

	if(t2.state!='working') return;
	if(hybrid!=1) return;
	phoneEU.CallNumber(cid, number);
}

function t2CmndFixedGetState(cid, hybrid)
{
	var ln={state:'NONE',remote:''};

	if(!(cid in t2Connections)) return ln;
	var t2=t2Connections[cid];

	if(t2.state!='working' || hybrid!=1) return ln;
	ln.state=t2.lineState; ln.remote=t2.remote;
	return ln;
}
